use crate::cli_args::SecretInput;
use crate::error::Fido2LuksResult;
use crate::luks::LuksDevice;
use std::io::{stdin, BufRead, BufReader};
use std::path::{Path, PathBuf};

enum Setup {
    Welcome,
    Disks {
        disks: Vec<PathBuf>,
    },
    Unlocked {
        disks: Vec<LuksDevice>,
        current_secrets: Vec<Vec<u8>>,
    },
    Salt {
        disks: Vec<PathBuf>,
        current_secrets: Vec<Vec<u8>>,
        salt: SecretInput,
    },
    Config {
        salt: SecretInput,
    },
}

impl Setup {
    fn description(&self) -> String {
        match self {
            Self::Welcome => {
                "Please enter the device path for all disks you want to protect".into()
            }
            _ => "".into(),
        }
    }

    fn next(self) -> Fido2LuksResult<Self> {
        println!("{}", self.description());
        Ok(match self {
            Self::Welcome => {
                let mut disks = Vec::new();
                for path in BufReader::new(stdin()).lines() {
                    let path_string = path?;
                    let path = match path_string.as_str() {
                        "" => break,
                        path => Path::new(path),
                    };
                    if path.exists() {
                        disks.push(path.to_path_buf());
                    } else {
                        eprintln!(
                            "{} does not exist, try again or hit enter to finish",
                            path.display()
                        );
                    }
                }
                Self::Disks { disks }
            }
            Self::Disks { disks } => {
                let mut luks_devs = Vec::new();
                let add_dev = |path: &Path| -> Fido2LuksResult<Option<LuksDevice>> {
                    match LuksDevice::load(path) {
                        Ok(luks) => return Ok(Some(luks)),
                        Err(dev) => {
                            let mut ans = String::new();
                            loop {
                                eprint!("Failed to open {}\nIs it an LUKS device or could it be the mounted block device? Skip/Replace (s,r): ");
                                stdin().read_line(&mut ans)?;
                                match ans.as_str() {
                                    "s" => return Ok(None),
                                    "r" => {
                                        print!("Enter an new device path: ");
                                        stdin().read_line(&mut ans)?;
                                        let path = PathBuf::from(&ans);
                                        return add_dev(path.as_path());
                                    }
                                    _ => (),
                                }
                            }
                        }
                    }
                    Ok(None)
                };
                for dev in disks.iter() {
                    let path = PathBuf::from(&dev);
                    if let Some(luks) = add_dev(path.as_path()) {
                        luks_devs.push(luks);
                    }
                }
                unreachable!()
            }
            _ => unreachable!(),
        })
    }
}
